@@ -117,12 +117,12 @@ def parse_scp(soup: BeautifulSoup, scp_id: Union[str, int]):
       # Use bold portions as keys/identifiers for their sections.
       if item.strong:
         key = item.strong.get_text(strip=True).rstrip(':')
-        value = str(item.strong.next_sibling).strip()
+        value = str(item.strong.next_sibling).strip(': ')
       
       else:
         # Add subsequent paragraphs to the same section.
         if key is not None:
-          value = mapping[key] + ' ' + item.get_text(strip=True)
+          value = f'{mapping[key]}\n{item.get_text(strip=True)}'
         
         # Don't if there's no section to add them to.
         else:
@@ -190,9 +190,10 @@ def get_scp(scp_id: Union[str, int]):
   parsed_content = parse_scp(site_content, int(scp_id))
 
   # Get SCP's name and add it to parsed_content.
-  if '[ACCESS DENIED]' not in get_scp_name(int(scp_id)) and get_scp_name(int(scp_id)) is not None:
-    scp_name = get_scp_name(int(scp_id))
-    parsed_content['name'] = scp_name
+  if get_scp_name(int(scp_id)) is not None:
+    if '[ACCESS DENIED]' not in get_scp_name(int(scp_id)):
+      scp_name = get_scp_name(int(scp_id))
+      parsed_content['name'] = scp_name
   
   # Don't add the name if there was an error preventing get_scp_name from grabbing it.
   else:
@@ -218,7 +219,7 @@ def get_scp_name(id: int):
 #     print(f"\nWARNING: Failed to scrape SCP-{id}! Error: {e}", file=sys.stderr)
     pass
 
-def scrape_scps(min_skip: int=0, max_skip: int=6000, ai_dataset: bool=False):
+def scrape_scps(min_skip: int=0, max_skip: int=6000, ai_dataset: bool=False, tags: list=[]):
   """
   Scrapes as much info on all SCPs from min_skip to max_skip - 1 as possible. Writes this info to different files based on its section.
 
@@ -228,7 +229,8 @@ def scrape_scps(min_skip: int=0, max_skip: int=6000, ai_dataset: bool=False):
   Parameters:
     min_skip: The SCP number to start at. Default: 0
     max_skip: The SCP number to end at plus one. Default: 6000
-    ai_dataset: Set to True if data is later going to be used to train an AI. Adds "\<\|endoftext\|\>" tokens where necessary to divide the dataset for training. Default: False
+    ai_dataset: Set to True if data is later going to be used to train an AI. Adds "<|endoftext|>" tokens where necessary to divide the dataset for training. Default: False
+    tags: The list of tags to grab from. Will ignore SCPs without these tags. An empty list (default) matches all tags.
   """
   # Create/clear the files we need for scraping.
   filelist = []
@@ -251,120 +253,142 @@ def scrape_scps(min_skip: int=0, max_skip: int=6000, ai_dataset: bool=False):
     else:
       j = i
     
-    # # Grab page content for each SCP in its own HTML file.
-    # # Useful for generating large HTML corpi.
-    # if i < 5000:
-    #   with open(f'z-scp-page-content-{j}.html', "w") as out:
-    #     soup = BeautifulSoup(urllib.request.urlopen(f'http://scp-wiki.net/scp-{j}'), 'html.parser')
-    #     content = soup.find('div', id='page-content')
-    #     out.write(str(content))
     try:
       # Get all the things for the SCP.
       mylist = get_scp(i)
-
+      
+      # Tag match checking code
+      match = False
+      
+      if tags != []:
+        for tag in tags:
+          if tag in mylist["tags"]:
+            match = True
+            break
+      
+      else:
+        match = True
+      
       # Get the list of keys in the dictionary (so we can search through it later).
-      keyslist = mylist["content"].keys()
-      try:
-        # Append current SCP's description to the description file.
-        with open('scp-descrips.txt', 'a') as out:
-          try:
-            out.write(f'Description: {mylist["content"]["Description"]}\n')
-            
-            # Add <|endoftext|> token if it's a dataset for training AI.
-            if ai_dataset:
-              out.write('<\|endoftext\|>\n\n')
+      if match:
+        keyslist = mylist["content"].keys()
 
-          # Error handling.
-          except Exception as e:
-            # print(f'Failed to grab the description of SCP-{j}! Please grab it yourself! Error: {e}')
-            pass
+        # Put stuff in a better format for the AI, if we're making a dataset for one
+        if ai_dataset:
+          for k in keyslist:
+            mylist["content"][k] = mylist["content"][k].replace('\n', ' ')
+            mylist["content"][k] = mylist["content"][k].replace(j, 'XXXX')
 
-        # Append current SCP's conprocs to the conproc file.
-        with open('scp-conprocs.txt', 'a') as out:
-          try:
-            for k in keyslist:
-              # Search keys for "Containment", output to conproc file if it matches.
-              if "containment" in k.lower():
-                out.write(f'Special Containment Procedures: {mylist["content"][k]}\n')
-                
-                # Add <|endoftext|> token if it's a dataset for training AI.
-                if ai_dataset:
-                  out.write('<\|endoftext\|>\n\n')
-          
-          # Error handling.
-          except:
-            # print(f'Failed to grab the conprocs of SCP-{j}! It is probably not an article with a standard format! Please grab them yourself!')
-            pass
-        
         try:
-          # Append current SCP's title to the title file (if we can grab it).
-          with open('scp-titles.txt', 'a') as out:
-            # Even more redundancy. I know. This is getting ridiculous.
-            if mylist["name"] is not None:
-              if "[ACCESS DENIED]" not in mylist["name"]:
-                out.write(f'SCP-XXXX: {mylist["name"]}')
+          # Append current SCP's description to the description file.
+          with open('scp-descrips.txt', 'a') as out:
+            try:
+              # Add <|endoftext|> token if it's a dataset for training AI.
+              if ai_dataset:
+                out.write('Description: {}\n<|endoftext|>\n'.format(mylist["content"]["Description"].replace(j, 'XXXX')))
+              else:
+                out.write(f'Description: {mylist["content"]["Description"]}\n')
+
+              out.write('\n')
+
+            # Error handling.
+            except Exception as e:
+              # print(f'Failed to grab the description of SCP-{j}! Please grab it yourself! Error: {e}')
+              pass
+
+          # Append current SCP's conprocs to the conproc file.
+          with open('scp-conprocs.txt', 'a') as out:
+            try:
+              for k in keyslist:
+                # Search keys for "Containment", output to conproc file if it matches.
+                if "containment" in k.lower():
+                  if ai_dataset:
+                    out.write('Special Containment Procedures: {}\n<|endoftext|>\n'.format(mylist["content"][k].replace(j, 'XXXX')))
+                  else:
+                    out.write(f'Special Containment Procedures: {mylist["content"][k]}\n')
+
+                  # Add <|endoftext|> token if it's a dataset for training AI.
+
+
+                  out.write('\n')
+
+            # Error handling.
+            except:
+              # print(f'Failed to grab the conprocs of SCP-{j}! It is probably not an article with a standard format! Please grab them yourself!')
+              pass
+
+          try:
+            # Append current SCP's title to the title file (if we can grab it).
+            with open('scp-titles.txt', 'a') as out:
+              # Even more redundancy. I know. This is getting ridiculous.
+              if mylist["name"] is not None:
+                if "[ACCESS DENIED]" not in mylist["name"]:
+                  if ai_dataset:
+                    out.write(f'SCP-XXXX: {mylist["name"]}\n')
+                  else:
+                    out.write(f'SCP-{j}: {mylist["name"]}\n')
+
+                # Handle nonexistent SCPs.
+                else:
+                  # print(f'SCP-{j} doesn\'t exist yet!')
+                  pass
 
-              # Handle nonexistent SCPs.
               else:
                 # print(f'SCP-{j} doesn\'t exist yet!')
                 pass
-            
-            else:
-              # print(f'SCP-{j} doesn\'t exist yet!')
-              pass
-        
-        # Error handling.
-        except Exception as e:
-          # raise e
-          # print(f'Failed to grab the title of SCP-{j}! Please grab it yourself! Error: {e}')
-          pass
-        
-        # Find and append addenda (if they exist) to the addenda file.
-        with open('scp-addenda.txt', 'a') as out:
-          try:
-            # Define list or dictionary depending on whether or not we need the keys.
-            if ai_dataset:
-              addendalist = []
-            else:
-              addendalist = {}
-            
-            for k in keyslist:
-              # Search keys for "Addendum", add to addendalist if it matches.
-              if "addendum" in k.lower():
-                if ai_dataset:
-                  addendalist.append(mylist["content"][k])
-              
-                # Do the same thing for non-dataset, also adding the keys.
-                else:
-                  addendalist.update({k: mylist["content"][k]})
-            
-            # Write addenda to addenda file.
-            if ai_dataset:
-              for k in addendalist:
-                buffer = k.strip(': ')
-                out.write(f'Addendum XXXX-XX: {buffer}<\|endoftext\|>')
-            
-            # Do the same for non-dataset.
-            else:
-              for k in addendalist.keys():
-                buffer = f'{k}{addendalist[k]}'
-                out.write(buffer)
 
           # Error handling.
           except Exception as e:
-            # print(f'Failed to grab the addenda of SCP-{j}! Please grab them yourself (if they exist)! Error: {e}')
+            # raise e
+            # print(f'Failed to grab the title of SCP-{j}! Please grab it yourself! Error: {e}')
             pass
-      
-      # More error handling.
+
+          # Find and append addenda (if they exist) to the addenda file.
+          with open('scp-addenda.txt', 'a') as out:
+            try:
+              # Define list or dictionary depending on whether or not we need the keys.
+              if ai_dataset:
+                addendalist = []
+              else:
+                addendalist = {}
+
+              for k in keyslist:
+                # Search keys for "Addendum", add to addendalist if it matches.
+                if "addendum" in k.lower():
+                  if ai_dataset:
+                    addendalist.append(mylist["content"][k])
+
+                  # Do the same thing for non-dataset, also adding the keys.
+                  else:
+                    addendalist.update({k: mylist["content"][k]})
+
+              # Write addenda to addenda file.
+              if ai_dataset:
+                for k in addendalist:
+                  buffer = k.strip(': ')
+                  out.write('Addendum XXXX-XX: {}\n<|endoftext|>\n\n'.format(buffer.replace(j, 'XXXX')))
+
+              # Do the same for non-dataset.
+              else:
+                for k in addendalist.keys():
+                  buffer = f'{k}: {addendalist[k]}'
+                  out.write(f'{buffer}\n\n')
+
+            # Error handling.
+            except Exception as e:
+              # print(f'Failed to grab the addenda of SCP-{j}! Please grab them yourself (if they exist)! Error: {e}')
+              pass
+
+        # More error handling.
+        except Exception as e:
+          # print(f'Failed to write the info for SCP-{j}! Error: {e}')
+          pass
+
+      # Wow, just look at all that error handling!
       except Exception as e:
-        # print(f'Failed to write the info for SCP-{j}! Error: {e}')
+        # print(f'Failed to grab the info for {i}! Error: {e}')
         pass
-    
-    # Wow, just look at all that error handling!
-    except Exception as e:
-      # print(f'Failed to grab the info for {i}! Error: {e}')
-      pass
-    # print(mylist)
+      # print(mylist)
 
   filelist_names = [
                     'scp-descrips.txt',
@@ -395,11 +419,18 @@ def scrape_scps(min_skip: int=0, max_skip: int=6000, ai_dataset: bool=False):
 
   # print("Done!")
 
-def scrape_scps_html(min_skip: int=0, max_skip: int=6000):
+def scrape_scps_html(min_skip: int=0, max_skip: int=6000, ai_dataset: bool=False, tags: list=[]):
   """
-  [IN DEVELOPMENT]
-  
   Scrapes the html code of SCPs min_skip to max_skip - 1.
+  
+  Output files:
+    scp-html.txt.
+
+  Parameters:
+    min_skip: The SCP number to start at. Default: 0
+    max_skip: The SCP number to end at plus one. Default: 6000
+    ai_dataset: Set to True if data is later going to be used to train an AI. Adds "<|endoftext|>" tokens where necessary to divide the dataset for training. Default: False
+    tags: The list of tags to grab from. Will ignore SCPs without these tags. An empty list (default) matches all tags.
   """
   # Create/reset text file
   with open('scp_html.txt', "w"):
@@ -409,7 +440,7 @@ def scrape_scps_html(min_skip: int=0, max_skip: int=6000):
   blank_page = '<div style="text-align: center;">\n<h1 id="toc0"><span>This page doesn\'t exist yet!</span></h1>\n</div>\n<hr>\n<div style="background-color: #600; border: solid 1px #600; border-radius: 20px; color: #fff; width: 450px; margin: 0 auto; font-size: 150%; text-align: center; box-shadow: 0 2px 6px rgba(0,0,0,.5), inset 0 1px rgba(255,255,255,.5), inset 0 10px rgba(255,204,204,.5), inset 0 10px 20px rgba(255,204,204,.3), inset 0 -15px 30px rgba(48,0,0,.5); line-height: 100%; padding: 0 10px;">\n<p><strong>Did you get feedback first?</strong></p>\n</div>\n<div style="background-color: #fff0f0; border: solid 1px #600; border-radius: 20px; color: #300; width: 450px; margin: 20px auto 0; text-align: center; box-shadow: 0 2px 6px rgba(0,0,0,.5); padding: 0 10px;">'
   
   for i in tqdm(range(min_skip, max_skip), "Fetching skips", total=max_skip, ncols=150, initial=min_skip, unit="skip", file=sys.stdout, bar_format='{desc}... {percentage:3.2f}% |{bar}|  [{remaining} remaining, {rate_fmt}]', smoothing=0.01875):
-    with open('scp_html.txt', "a") as out:
+    with open('scp-html.txt', "a") as out:
       if i < 10:
         j = f'00{i}'
       elif i < 100:
@@ -420,26 +451,32 @@ def scrape_scps_html(min_skip: int=0, max_skip: int=6000):
       soup = get_single_scp(j)
       
       if soup is not None:
-        content = soup.find('div', id='page-content')
+        # Get page tags
+        tags_list = soup.find('div', {'class': 'page-tags'}).find('span')
+        page_tags = [tag.string for tag in tags_list if tag.string != '\n']
 
-        if blank_page not in content:
-          out.write(f'{content}<\|endoftext\|>\n')
+        # Tag match checking code
+        match = False
 
+        if tags != []:
+          for tag in tags:
+            if tag in page_tags:
+              match = True
+              break
+        
         else:
-          # print(f'\nThe page for SCP-{j} is blank!', file=sys.stderr)
-          pass
-    
-    if i % 250 == 0 and i != 0:
-      try:
-        get_ipython().system('zip -9 -uq html_scraper_output.zip scp_html.txt')
+          match = True
+        
+        if match:
+          content = soup.find('div', id='page-content')
 
-      except NameError:
-        import zipfile
+          if blank_page not in content:
+            if ai_dataset:
+              out.write('{}\n\n<|endoftext|>\n\n\n'.format(str(content).replace(j, 'XXXX')))
 
-        if sys.version_info[0] == 3 and sys.version_info[1] >= 7:
-          with zipfile.ZipFile('html_scraper_output.zip', "w", compression=zipfile.ZIP_DEFLATED, compresslevel=9) as zip_out:
-            zip_out.write('scp_html.txt')
+            else:
+              out.write(f'{content}\n\n')
 
-        elif sys.version_info[0] == 3:
-          with zipfile.ZipFile('html_scraper_output.zip', "w", compression=zipfile.ZIP_DEFLATED) as zip_out:
-            zip_out.write('scp_html.txt')
+          else:
+            # print(f'\nThe page for SCP-{j} is blank!', file=sys.stderr)
+            pass
